<?php
/*
Plugin Name: MiniMU
Plugin URI: http://www.smashlab.com/incubator/minimu
Description: Manage multiple blogs with a single standard WordPress installation. Each may have its own theme and domain while sharing users and administration.
Version: 0.3.1
Author: Eric Shelkie
Author URI: http://www.smashlab.com
License: GPL2
*/

/*  Copyright 2010 Eric Shelkie (email : shelkie@smashlab.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// error_reporting(E_ALL);
// ini_set('display_errors',true);

class MiniMU{
	const VERSION="0.3.1";
	const WP_OPTION_KEY='minimu-options';
	const DEFAULT_CATEGORY_BASE="category";
	const MAX_CUSTOM_VAR_NAME_LENGTH=16;
	protected $arr_plugin_options=array();
	private static $instance;
	static $in_admin=false;
	protected $plugin_directory;
	public $arr_domains;
	protected $arr_categories;
	protected $arr_themes;
	protected $bln_settings_saved=false;
	public $current_domain_id=null;
	protected $obj_current_domain_options;
	protected $base_domain;
	protected $obj_base_domain;
	protected $arr_values=array(
		array('input_name'=>'minimu-domain-name', 'property_name'=>'domain_name'),
		array('input_name'=>'minimu-catselect', 'property_name'=>'category_id'),
		array('input_name'=>'minimu-themeselect', 'property_name'=>'theme_name'),
		array('input_name'=>'minimu-blog-title', 'property_name'=>'blog_title'),
		array('input_name'=>'minimu-tagline', 'property_name'=>'tagline'),
		array('input_name'=>'minimu-custom-vars', 'property_name'=>'custom_vars')
	);

	/**
	 * Get instance of this class
	 *
	 * @return MiniMU
	 */
	static function get_instance(){
		if (!self::$instance instanceof MiniMU ){
			self::$instance=new MiniMU();
		}
		return self::$instance;
	}

	/**
	 * Initialize plugin
	 *
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	public function __construct(){
		global $minimu_obj_base_domain;
		$this->base_domain=$minimu_obj_base_domain->domain_name;

		// Add filters
		add_filter('option_url', 'MiniMU::filter_url', 0);  /************ this s/b removed? ****************/
		add_filter('site_url', 'MiniMU::filter_site_url', 0,3);
		add_filter('option_blogname', 'MiniMU::filter_blogname', 0);
		add_filter('option_home', 'MiniMU::filter_home', 0);
		add_filter('option_blogdescription', 'MiniMU::filter_blogdescription', 0);
		add_filter('option_template', 'MiniMU::filter_template', 0);
		add_filter('option_stylesheet', 'MiniMU::filter_stylesheet', 0);
		//add_filter('the_posts', 'MiniMU::the_posts');
		add_filter('post_link', 'MiniMU::filter_post_link', 0, 3);
		add_filter('getarchives_where', 'MiniMU::filter_getarchives_where');
		add_filter('getarchives_join', 'MiniMU::filter_getarchives_join' );
		add_filter('posts_where', 'MiniMU::filter_posts_where');
		add_filter('posts_join', 'MiniMU::filter_posts_join' );

		
		
		add_filter('body_class','MiniMU::filter_body_class');
		add_filter('category_link','MiniMU::filter_category_link', 0, 2);
		add_filter('the_content', 'MiniMU::filter_replace_tokens');
		add_filter('the_excerpt', 'MiniMU::filter_replace_tokens');


		// Set up admin menus
		add_action( 'admin_init', 'MiniMU::admin_init' );
		add_action('admin_menu', 'MiniMU::add_admin_menu');		//$this->plugin_directory=WP_PLUGIN_URL.'/'.str_replace(basename( __FILE__),"",plugin_basename(__FILE__));

		// Get plugin options
		$this->arr_domains=array();
		$this->arr_plugin_options=$this->get_plugin_options();
		
		if (is_array($this->arr_plugin_options) && isset($this->arr_plugin_options['arr_domains'])) $this->arr_domains=$this->arr_plugin_options['arr_domains'];
		if (!isset($this->arr_domains['0'])) $this->arr_domains['0']=$minimu_obj_base_domain;

		// The wordpress base/default domain
		$this->obj_base_domain=$minimu_obj_base_domain;
		$this->arr_domains['0']=(object)array_merge((array)$this->arr_domains['0'], (array)$this->obj_base_domain);
	}
	
	
	/**
	 * Get plugin options
	 *
	 * @return array $plugin_options
	 * @author Eric Shelkie
	 * @since December 7, 2009
	 */
	protected function get_plugin_options(){
		// Check if we already have the options
		if (is_array($this->arr_plugin_options) && isset($this->arr_plugin_options['arr_domains'])) return $this->arr_plugin_options;
		
		// Get options from database
		$arr_options=get_option(MiniMU::WP_OPTION_KEY);
		if (is_array($arr_options) && isset($arr_options['version'])){
				
			// Check if the options match the plugin, and if not, perform any necessary updates
			if ($arr_options['version']<MiniMU::VERSION){
				// Additional version checks will go here, as required
				
				// Save updated options array
				$arr_options['version']=MiniMU::VERSION;	
				update_option(MiniMU::WP_OPTION_KEY, $arr_options);
			}
		}else{
			// Didn't find any saved options
			$arr_options=array();
			$arr_options['version']=MiniMU::VERSION;	
			$arr_options['arr_domains']=array();

			// See if there are other options in the database
			if ($old_options=get_option('minimu_arr_domains')){
				$arr_options['arr_domains']=$old_options;
			}
			update_option(MiniMU::WP_OPTION_KEY, $arr_options);
		}
		$this->arr_plugin_options=$arr_options;
		return $this->arr_plugin_options;
	}


	/**
	 * Get options for domain
	 *
	 * @param int $domain_id
	 * @return stdClass Options
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function get_domain_options($domain_id=null){
		$return_value=false;
		$dc_instance=MiniMU::get_instance();

		if (is_null($domain_id)) $domain_id=$dc_instance->get_current_domain_id();
		if (!$domain_id) $domain_id=0;
		if ($domain_id!==false){
			if (isset($dc_instance->arr_domains[$domain_id])){
				$return_value=$dc_instance->arr_domains[$domain_id];
			}
		}	
		return $return_value;
	}


	/**
	 * Get the domain id of the current URI
	 *
	 * @return int Domain id
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	private function get_current_domain_id(){
		// Check if we've already done this
		if (!is_null($this->current_domain_id)) return $this->current_domain_id;

		$arr_domains=$this->get_domain_array_by_domain_name();
		$protocol=(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']!='off')?'https://':'http://';
		
		// Try to match the page to the list of configured domains
		$arr_server_path_options=array(
			$_SERVER['SERVER_NAME'],
			$_SERVER['HTTP_HOST'],
			trim($_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI'],' /'),
			trim($_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'], ' /')
		);

		// Look through possible domain matches, and use the one with the most "/" characters
		$arr_match=array_intersect($arr_domains, $arr_server_path_options);
		//echo "<pre>" . print_R($arr_match, true) . "</pre>";
		if (is_array($arr_match) && count($arr_match)>0){
			$max=0;
			$best_match=null;
			foreach (array_reverse($arr_match) as $key=>$value){
				$count=substr_count($value, '/');
				if ($count>=$max){
					// Use the base domain if its domain name is used by additional minimu domains
					if ($count==$max && !is_null($best_match) && $this->arr_domains[$best_match]->is_base) continue;
					
					$max=$count;
					$best_match=$key;
				}
			}
			$key=$best_match;
		}					
		
		// See if the URI contains category info - this would override the domain match
		$arr_page_categories=array();
		
		// Try to get category from current uri
		$arr_temp=explode('?', trim($_SERVER['REQUEST_URI']));
		
		$path=trim($arr_temp[0], '/');
		
		// clean up path
		$category_base=trim(get_option('category_base'),' /');
		if ($category_base=='') $category_base=MiniMU::DEFAULT_CATEGORY_BASE;
		$path='/' . str_replace($category_base, '', $path);
		
		// use WP function to find category in path
		$obj_category=get_category_by_path($path, true);

		if ($obj_category instanceof stdClass ){
			$arr_page_categories[]=$obj_category->cat_ID;
		}else{
		
			// try to get category from REQUEST
			if (isset($_REQUEST['cat'])) $arr_page_categories[]=(int) $_REQUEST['cat'];
		}

		// see if we can match the current category with one in our domain list
		if (count($arr_page_categories)>0){
			
			// Okay, we have some categories to consider. If none match though, we'll default to the base domain
			$key=0;
			
			// Try to match the category ID to a domain
			if ($match=$this->get_domain_from_category_id($arr_page_categories)) $key=$match->id;
		}

		// Compare current url to detected domain and redirect if necessary
		if (isset($this->arr_domains[$key])){
			$obj_domain=$this->arr_domains[$key];
			$domain_match=array_intersect($arr_server_path_options, array($obj_domain->domain_name));
			if (count($domain_match)==0){
				global $query;
				$current_url = $protocol . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
				
				if (!stristr($current_url, $obj_domain->domain_name)){
					$new_url=str_replace($_SERVER['HTTP_HOST'] , $obj_domain->domain_name, $current_url);
					//wp_redirect($new_url, 301);
				}
			}
		}

		$this->current_domain_id=$key;
		return $key;
	}
	
	

	/**
	 * Try to find a configured domain that is linked to a category
	 *
	 * @param array $arr_category_ids
	 * @return object domain
	 * @author Eric Shelkie
	 * @since December 16, 2009
	 */
	private function get_domain_from_category_id($arr_category_ids){
		if (!is_array($arr_category_ids)) $arr_category_ids=array($arr_category_ids);
		$arr_domains=$this->get_domain_array_by_domain_name();
		$obj_return=null;
		
		// Loop through the configured domains and use the first one who's category matches
		foreach($arr_domains as $domain_id=>$str_domain){
			if (isset($this->arr_domains[$domain_id])){			
				$obj_domain=$this->arr_domains[$domain_id];
				if (in_array($obj_domain->category_id, $arr_category_ids)){
					$obj_return=$obj_domain;
					break;
					
				}
			}	
		}
		return $obj_return;
	}


	/**
	 * Get an array of configured domains with id as key and domain as value
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */	
	protected function get_domain_array_by_domain_name(){
		$arr_return=array();
		foreach ($this->arr_domains as $key=>$obj_domain){
			if ($obj_domain instanceof stdClass && isset($obj_domain->domain_name)) {
				$arr_return[$key]=MiniMU::get_token('domain_name', $obj_domain->id);
			}
		}
		return $arr_return;
	}


	/**
	 * Filter to replace url
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function filter_site_url($str_input, $path, $scheme){
		$dc_instance=MiniMU::get_instance();
		$str_output=$str_input;
		$domain_name=$str_input;
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options->is_base) return $str_input;
		if ($obj_options instanceof stdClass ){
			//$protocol=(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']!='off')?'https://':'http://';
			$old_domain=trim(str_replace($path, '', $str_input),'/ ');
			$old_domain=preg_replace("/http[s]?:\/\//i", '', $old_domain);
			$new_domain=trim(MiniMU::get_token('domain_name'), '/ ');
			$new_url=str_replace($old_domain, $new_domain, $str_input);
			$str_output=$new_url;
		}
		return $str_output;
	}


	/**
	 * Filter to replace siteurl
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function filter_home($str_input){
		$dc_instance=MiniMU::get_instance();
		$str_output=$str_input;
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options->is_base) return $str_input;
		if ($obj_options instanceof stdClass ){
			$protocol=(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS']!='off')?'https://':'http://';
			$domain_name=MiniMU::get_token('domain_name');
			if ($domain_name) $str_output=$protocol . $domain_name;
		}
		return $str_output;
	}

	/**
	 * Filter to replace Blog title
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function filter_blogname($str_input){
		$dc_instance=MiniMU::get_instance();
		$str_output=$str_input;
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options->is_base) return $str_input;
		if ($obj_options instanceof stdClass ){
			if (isset($obj_options->blog_title)) $str_output=$obj_options->blog_title;
		}
		return $str_output;
	}
	
	/**
	 * Custom SQL join for the_posts
	 * 
	 * @param string $str_join
	 * @return string @str_join modified
	 * @author Eric Shelkie
	 * @since March 14, 2010
	 */
	static function filter_posts_join( $str_join ) {
		global $wpdb;
		return $str_join . " INNER JOIN $wpdb->term_relationships as minimu_relationships ON ($wpdb->posts.ID = minimu_relationships.object_id) INNER JOIN $wpdb->term_taxonomy as minimu_term_taxonomy ON (minimu_relationships.term_taxonomy_id = minimu_term_taxonomy.term_taxonomy_id)";
	}
	
	
	/**
	 * Add post archive filter - should not show posts that are in a different domain/category
	 *
	 * @param string $where clause
	 * @return string $where clause
	 * @author Eric Shelkie
	 * @since March 15, 2010
	 */
	static function filter_posts_where($str_where){
		global $wpdb;
		
		$dc_instance=MiniMU::get_instance();
		$obj_options=$dc_instance->get_domain_options();
		$arr_other_categories=array();
		$str_where_modified=$str_where;
		
		foreach ($dc_instance->arr_domains as $obj_domain){
			if ($obj_domain->id!=$dc_instance->current_domain_id){
				if ($obj_domain->category_id!=$obj_options->category_id){
					$arr_other_categories[]=$obj_domain->category_id;
				}
			}
		}
		
		if (count($arr_other_categories)>0){
			$str_exclude=implode(',', $arr_other_categories);
		
			$str_where_modified.=" AND minimu_term_taxonomy.taxonomy = 'category' AND minimu_term_taxonomy.term_id NOT IN ($str_exclude)";
		}
		
		return $str_where_modified;
	}
	
	
	/**
	 * Filter to only show posts for the associated category
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	/*
	static function the_posts($arr_posts){
		$arr_return=array();
		$dc_instance=MiniMU::get_instance();
		
		// Get the options for the current domain
		$obj_options=$dc_instance->get_domain_options();
		
		// If no category has been assiged to this domain, return all posts
		if (!$obj_options->category_id || MiniMU::$in_admin) return $arr_posts;

		// Check each post to see if it should be included
		foreach ($arr_posts as $post){
			$arr_post_categories=wp_get_post_categories($post->ID);
			if (in_array($obj_options->category_id, $arr_post_categories)) $arr_return[]=$post;
		}
		return $arr_return;
	}
	*/
	
	
	/**
	 * Filter category links
	 * 
	 * @param string $str_link
	 * @param stdClass $int_category
	 * @return string $permalink
	 * 
	 * @author Eric Shelkie
	 * @since December 17, 2009
	 */
	static function filter_category_link($str_link, $int_category=null){
		//$arr_domain=MiniMU::
		$dc_instance=MiniMU::get_instance();
		$new_domain=$dc_instance->obj_base_domain->domain_name;
		if ($int_category){
			$obj_domain=$dc_instance->get_domain_from_category_id($int_category);
			if ($obj_domain instanceof stdClass ){
				$new_domain=$obj_domain->domain_name;
			}
		}
		//echo "$str_link $int_category - $new<br />";
		if ($new_domain && !strstr($str_link, $new_domain)){
			//echo "siteURL: " . site_url() . "<br />";
			$old_domain=preg_replace("/http[s]?:\/\//",'',site_url());
			$new_link=str_replace($old_domain, $new_domain, $str_link);
			//echo "rewrite link: $str_link to $new_link -- $old_domain > " . $obj_domain->domain_name .  "<br />";
			$str_link=$new_link;
		}
		return $str_link;
	}
	
	
	/**
	 * Filter post links
	 * 
	 * @param string $permalink
	 * @param stdClass $post
	 * @param string $leavename
	 * @return string $permalink
	 * 
	 * @author Eric Shelkie
	 * @since December 19, 2009
	 */
	static function filter_post_link($permalink, $post=null, $leavename=null){
		$arr_category_ids=array();
		$dc_instance=MiniMU::get_instance();
		if ($post instanceof stdClass){
			$arr_post_categories=get_the_category($post->ID);
			foreach($arr_post_categories as $obj_category) $arr_category_ids[]=$obj_category->cat_ID;

			// If the post belongs to the current category, then don't change anything
			if (!in_array(the_category_ID(false), $arr_category_ids)){
				
				// otherwise, see if the post belongs to a configured category
				$match=null;
				foreach ($dc_instance->arr_domains as $obj_domain){
					if (($match=array_search($obj_domain->category_id, $arr_category_ids))!==false){
						$match=$arr_category_ids[$match];
						break;
					}
				}
				
				// Transform the link to reference to correct category
				$permalink=$dc_instance->filter_category_link($permalink, $match);
			}
		}
		return $permalink;
	}

	/**
	 * Filter to replace Tagline
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function filter_blogdescription($str_input){
		$dc_instance=MiniMU::get_instance();
		$str_output=$str_input;
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options->is_base) return $str_input;
		if ($obj_options instanceof stdClass ){
			if (isset($obj_options->tagline)) $str_output=$obj_options->tagline;
		}
		return $str_output;
	}

	/**
	 * Filter to change template
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function filter_template($str_input){
		$dc_instance=MiniMU::get_instance();
		$str_output=$str_input;
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options->is_base) return $str_input;
		if ($obj_options instanceof stdClass ){
			if (isset($obj_options->theme_name)){
				$arr_themes=$dc_instance->get_themes_array();
				if (isset($arr_themes[$obj_options->theme_name])){
					$str_output=$arr_themes[$obj_options->theme_name]->Template;
				}
			}
		}
		return $str_output;
	}

	/**
	 * Filter to change stylesheet
	 *
	 * @author Eric Shelkie
	 * @since November 26, 2009
	 */
	static function filter_stylesheet($str_input){
		$dc_instance=MiniMU::get_instance();
		$str_output=$str_input;
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options instanceof stdClass ){
			if (isset($obj_options->theme_name)){
				$arr_themes=$dc_instance->get_themes_array();
				if (isset($arr_themes[$obj_options->theme_name])){
					$str_output=$arr_themes[$obj_options->theme_name]->Stylesheet;
				}
			}
		}
		return $str_output;
	}
	
	
	/**
	 * Add the category slug to the body class so we can style categories more easily
	 *
	 * @param array body classes
	 * @return array modified body class
	 * @author Eric Shelkie
	 * @since November 30, 2009
	 */
	static function filter_body_class($arr_body_classes){
		$dc_instance=MiniMU::get_instance();
		
		$str_slug=MiniMU::get_domain_slug();
		if ($str_slug ){
			if (!in_array($str_slug, $arr_body_classes)) $arr_body_classes[]=$str_slug;
		}
		return $arr_body_classes;
	}
	
	
	/**
	 * Custom SQL join for getarchives
	 * 
	 * @param string $str_join
	 * @return string @str_join modified
	 * @author Eric Shelkie
	 * @since March 14, 2010
	 */
	static function filter_getarchives_join( $str_join ) {
		global $wpdb;
		return $str_join . " INNER JOIN $wpdb->term_relationships as minimu_relationships ON ($wpdb->posts.ID = minimu_relationships.object_id) INNER JOIN $wpdb->term_taxonomy as minimu_term_taxonomy ON (minimu_relationships.term_taxonomy_id = minimu_term_taxonomy.term_taxonomy_id)";
	}
	
	/**
	 * Add post archive filter - should not show posts that are in a different domain/category
	 *
	 * @param string $where clause
	 * @return string $where clause
	 * @author Eric Shelkie
	 * @since March 15, 2010
	 */
	static function filter_getarchives_where($str_where){
		global $wpdb;
		
		$dc_instance=MiniMU::get_instance();
		$obj_options=$dc_instance->get_domain_options();
		$arr_other_categories=array();
		$str_where_modified=$str_where;
		
		
		foreach ($dc_instance->arr_domains as $obj_domain){
			//echo "check " . print_r($obj_domain, true);
			if ($obj_domain->id!=$dc_instance->current_domain_id){
				if ($obj_domain->category_id!=$obj_options->category_id){
					$arr_other_categories[]=$obj_domain->category_id;
				}
			}
		}
		if (count($arr_other_categories)>0){
			$str_exclude=implode(',', $arr_other_categories);
			$str_where_modified.=" AND minimu_term_taxonomy.taxonomy = 'category' AND minimu_term_taxonomy.term_id NOT IN ($str_exclude)";
		}
		return $str_where_modified;
	}



	/**
	 * Add the admin menu items
	 *
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	static function add_admin_menu(){
		if ( is_admin() ){ // admin actions
			add_options_page('MiniMU options', 'MiniMU', 'administrator', 'wp-minimu-admin-main', 'MiniMU::options_page');
		} else {
		  // non-admin enqueues, actions, and filters
		}
	}


	/**
	 * Initialize Admin area
	 *
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	static function admin_init(){
		$dc_instance=MiniMU::get_instance();

		MiniMu::$in_admin=true;
		// Check if we're in the admin area and redirect to the base domain if necessary
		$site_url=preg_replace("/http[s]?\:\/\//i", '', $dc_instance->base_domain);
		$protocol=(is_ssl())?"https://":"http://";
		//echo "redirect to: " . $site_url . $_SERVER['REQUEST_URI'];
		if (!stristr($site_url, $_SERVER['SERVER_NAME']) && (isset($_SERVER['HTTP_HOST']) && !stristr( $site_url, $_SERVER['HTTP_HOST']))){
			wp_redirect($protocol . $site_url . $_SERVER['REQUEST_URI']);
			exit();
		}

		$dc_instance=MiniMU::get_instance();
		$arr_categories=$dc_instance->get_categories_array();
		$arr_themes=$dc_instance->get_themes_array();

		add_action('admin_head', 'MiniMU::minimu_head');
		add_action('wp_ajax_minimu_add_domain_click', 'MiniMU::add_domain_click');
		wp_enqueue_script( array("jquery", "jquery-ui-core", "interface", "jquery-ui-sortable", "wp-lists", "jquery-ui-sortable") );

		if (isset($_POST['minimu-action'])){

			switch($_POST['minimu-action']){

				case 'update':
					$dc_instance->save_options();
					break;
			}
		}
	}


	/**
	 * Add header JavaScript
	 *
	 * @author Eric Shelkie
	 * @since November 24, 2009
	 */
	static function minimu_head() {
		$arrCustomVarNames=MiniMU::get_custom_variable_names();
		?>
		<script type="text/javascript">
			var minimu_custom_var_names=Array(<?php if(count($arrCustomVarNames)>0) echo '"' . implode('","', $arrCustomVarNames) . '"';?>);
		
			function minimu_post_ajax(data){
				var ajax_url = '<?php echo admin_url("admin-ajax.php"); ?>';

				jQuery.post(ajax_url, data, function(str_response) {
					eval("response=" + str_response);
					if (response.command){
						eval(response.command + "(response.args)");
					}
				});
			}

			function minimu_add_empty_domain(args){
				jQuery(".minimu-domain-container:last").after(args);
				smpSortableInit();
				minimu_attach_events();
			}
			
			function minimu_add_var(obj_domain){
				var new_varname=prompt("New token name:");
				if (new_varname){
					var invalid=new_varname.match(/[^a-zA-Z\-\_]/);
					new_varname=new_varname.substr(0, <?php echo MiniMU::MAX_CUSTOM_VAR_NAME_LENGTH;?>);
					if (new_varname.length<1) invalid=true;
				 	if (invalid){
				 		alert("Invalid name! Please use a-z, A-Z, underscore, dash");
				 		return minimu_add_var(obj_domain);
				 		
				 	}else if(jQuery.inArray(new_varname.toLowerCase(),  jQuery.map(minimu_custom_var_names, function(item){return item.toLowerCase;}))>=0){
						alert("That token name is already used!");
						return minimu_add_var(obj_domain);	
				 	}else{
				 		minimu_insert_var(obj_domain, new_varname);
				 		minimu_custom_var_names.push(new_varname);
				 		minimu_attach_events();
					}
				 }
			}
			
			
			function minimu_insert_var(obj_domain, var_name){
				id_match=obj_domain.attr('id').match(/\-(\d*)$/);
				domain_id=id_match[1];
				if (domain_id){
					obj_domain.find(".minimu-add-var-row").before('<tr class="minimu-var-row"><td class="c0"><a class="remove" href="Javascript:{}">X</a></td><td class="c1">' + var_name + '</td><td class="c2"><textarea name="minimu-custom-vars[' + domain_id + '][' + var_name + ']" class="minimu-var-textarea"></textarea></td></tr>');
					minimu_attach_events();
				}
			}
			
		 
			var smpSortable;
			function smpSortableInit() {
				  try { // a hack to make sortables work in jQuery 1.2+ and IE7
				   jQuery('#sortcontainer').SortableDestroy();
				  } catch(e) {}
				  smpSortable = jQuery('#sortcontainer').Sortable( {
				  accept: 'sortable',
				  onStop: smpSortableInit,
				  handle: '.sortable-handle',
				  axis: 'vertical'
				  } );
			 }

			 // Add delete domain button event
			function minimu_delete_domain(domain_id){
				var confirmed=confirm("Are you sure you want to remove this domain?");
				if (confirmed){
					jQuery("#minimu-tr-" + domain_id).fadeOut(function(){
						jQuery(this).remove();
					});
					jQuery("#minimu-delete").attr("value", jQuery("#minimu-delete").attr("value") + "," + domain_id);
				}
			 }
		 

			 function minimu_attach_events(){
				 // Show delete variable button on hover
				 jQuery(".minimu-domain-container .minimu-var-row").unbind("hover");
				 jQuery(".minimu-domain-container .minimu-var-row").hover(function(){
					jQuery(this).find(".remove").show();
				 },function(){
					 jQuery(this).find(".remove").hide();
				 });

				 // Add delete variable button event
				 jQuery(".minimu-domain-container .minimu-var-row .c0 .remove").unbind("click");
				 jQuery(".minimu-domain-container .minimu-var-row .c0 .remove").click(function(){
					var confirmed=confirm("Are you sure you want to remove this value?");
					if (confirmed){
						jQuery(this).parents(".minimu-var-row").fadeOut(function(){
							jQuery(this).remove();
						});
					}
				 });


				// Custom event that update select options
				 jQuery('.minimu-add-var-select').bind('updateOptions', function(){
					var obj_select=jQuery(this);
					obj_select.children().remove();
				 	var output = Array('<option value="" >Select...</option>');
				 	jQuery.each(minimu_custom_var_names, function(key, value){
						if (obj_select.parents(".minimu-domain-container").find("[name$='][" + value + "]']").length==0){
				 	  		output.push('<option value="'+ value +'">'+ value +'</option>');
						}
				 	});
				 	output.push('<option value="new" >Add new...</option>');
				 	obj_select.get(0).innerHTML = output.join('');
				 });

				 jQuery(".minimu-add-var-link").unbind("click");
				 jQuery(".minimu-add-var-link").click(function(e){
					 	var obj_select=jQuery(this).siblings('.minimu-add-var-select');
					 	var obj_domain=jQuery(this).parents(".minimu-domain-container");
					 	var val;
					 	if (obj_select.attr('options').length==1){
					 		// Add new variable
					 		minimu_add_var(obj_domain);
					 	}else{
						 	jQuery(this).hide();
						 	obj_select.trigger('updateOptions');
						 	obj_select.show();
						}
					 });
					 
					 /**
					  * Select a custom variable, or add a new one
					  *
					  */
					 jQuery(".minimu-add-var-select").unbind("change");
					 jQuery(".minimu-add-var-select").change(function(e){
					 	var valid=false;
					 	var new_varname;
					 	var select=jQuery(this);
					 	var obj_domain=jQuery(this).parents(".minimu-domain-container");
					 	var val=select.val();
					 	
					 	select.hide();
					 	select.attr("selectedIndex", 0);
					 	select.siblings('.minimu-add-var-link').show();
					 	if (val){
					 		if (val=='new'){
					 			minimu_add_var(obj_domain);
					 		}else{
					 			minimu_insert_var(obj_domain, val);
					 		}
					 	}

					 });

					 jQuery(".minimu-add-var-select").unbind("blur");
					 jQuery(".minimu-add-var-select").blur(function(e){
					 	jQuery(this).hide();
					 	jQuery(this).siblings('.minimu-add-var-link').show();
					 });
					 
			 }
		 
		
			jQuery(document).ready(function(){
				 // initialize sortable
				 smpSortableInit();

				 minimu_attach_events();
				 
			});
			
		</script>
		<style type="text/css">
		<!--
			.minimu-domain-container{
				width: 500px;
			}
			.minimu-domain-container input, .minimu-domain-container select{
				width: 340px;
			}
			
			.minimu-domain-container select {
				font-size:12px !important;
				height:2em;
				padding:2px;
			}
			
			.minimu-domain-container td.c0{
				width: 25px;
				vertical-align: top;
				text-align: left;
			}
			
			.minimu-domain-container td.c0 a.remove{
				float: left;
				width: 1em;
				height: 1em;
				margin: 5px 3px 0 3px;
				padding: 2px 0px 2px 2px;
				line-height: 1em;
				border: 1px solid #999;
				background-color: #999;
				text-decoration: none;
				color: #fff;
				font-weight: bold;
				font-size: 0.6em;
			}
			
			.minimu-domain-container td.c0 a.remove: hov:hover{
				text-decoration: none;
			}
			
			.minimu-domain-container .minimu-var-row a.remove{
				display: none;
			}
			
			
			.minimu-domain-container td.c1{
				width: 100px;
				vertical-align: top;
				padding: 0.4em 2px 0 10px;
				text-align: right;
			}
			
			.minimu-domain-container td.c2{
				width: 340px;
			}
			
			.minimu-domain-container td.c3{
				width: 10px;
			}
			
			.minimu-var-textarea{
				width: 340px;
			}
			
			select.minimu-add-var-select{
				width: 100px;
				display: none;
			}
			
			
			 .sortable-handle{
			 	width: 15px;
			 	background-color: #ccc;
			 }
			 

		-->
		</style>
		<?php

	}


	/**
	 * Save options
	 *
	 * @return void
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	protected function save_options(){
		$has_errors=false;
		if (is_admin()){
			if (isset($_POST['minimu-domain-name']) && is_array($_POST['minimu-domain-name'])){
				$arr_raw_domains=MiniMU::get_domain_objects_from_post();
				$this->arr_domains=array();

				foreach ($arr_raw_domains as $key=>$obj_domain){
					$obj_validated=null;
					$is_base=false;
					if (isset($obj_domain->is_base) && $obj_domain->is_base) $is_base=true;
					//print_r($obj_domain);
					$obj_validated=MiniMU::validate_domain($obj_domain);
					$obj_domain_safe=$obj_validated->sanitized;
					//print_R($obj_validated);

					$this->arr_domains[$key]=$obj_domain_safe;
					if ($obj_validated instanceof stdClass && count((array)$obj_validated->errors)>0) $has_errors=true;

					//echo "save: $key - " . $obj_domain_safe->domain_name;
				}
				
				// See if domains should be removed
				if (isset($_POST['minimu-delete'])){
					$arr_delete=explode(',',trim($_POST['minimu-delete'], ', '));
					foreach($arr_delete as $delete_id){
						if ($delete_id) unset($this->arr_domains[$delete_id]);
					}
				}

				// If there are no errors, go ahead and save
				if (!$has_errors){
					$this->arr_plugin_options['arr_domains']=$this->arr_domains;
					$this->arr_plugin_options['version']=MiniMU::VERSION;
					update_option(MiniMU::WP_OPTION_KEY, $this->arr_plugin_options);
					$this->bln_settings_saved=true;
				}
			}
		}
		// Clear domain array
		//update_option('minimu_arr_domains', array());
	}


	/**
	 * Extract domain objects from $_POST
	 *
	 * @return array Domain objects
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	protected function get_domain_objects_from_post(){
		$dc_instance=MiniMU::get_instance();
		$arr_raw_domains=array();
		if (isset($_POST['minimu-domain-name']) && is_array($_POST['minimu-domain-name'])){
			foreach ($_POST['minimu-domain-default'] as $key=>$is_default){
				$obj_domain=new stdClass();
				
				if ($is_default=='true'){
					//echo "Add default: " . print_r($dc_instance->obj_base_domain);
					$domain_id=0;
					$obj_domain=clone $dc_instance->obj_base_domain;
				}else{
					// if key==new, we're adding a new domain, so get the next free domain_id
					if (stristr($key, "new-")){
						if (count($dc_instance->arr_domains)>0){
							$domain_id=max(array_keys($dc_instance->arr_domains))+1;
							// Add an empty placeholder
							$dc_instance->arr_domains[$domain_id]=new stdClass();
						}else{
							$domain_id=1;
						}
					}else{
						$domain_id=(int)$key;
					}
				}
				
				// Add domain to the list
				$obj_domain->id=$domain_id;
				
				// Get form values and assign to object properties
				foreach($dc_instance->arr_values as $arr_value){
					$value=null;
					if (isset($_POST[$arr_value['input_name']][$key])){
						$value=$_POST[$arr_value['input_name']][$key];
						
						// Get custom variables
						if ($arr_value['property_name']=='custom_vars'){
							if (is_array($value)){
								foreach($value as $custom_var_name=>$custom_var_value){
									$key=trim(substr(MiniMU::sanitize_varname($custom_var_name), 0, MiniMU::MAX_CUSTOM_VAR_NAME_LENGTH));
									if ($key){
										$obj_value=new stdClass();
										$obj_value->value=$custom_var_value;
										$obj_value->type='textarea'; // might add more types
										$value[$key]=$obj_value;
									}
								}
							}
						}
					}
					$obj_domain->$arr_value['property_name']=$value;
				}

				// If this is the default domain, merge in the correct settings
				if ($is_default=='true'){
					$obj_domain=(object)array_merge((array)$obj_domain, (array)$dc_instance->obj_base_domain);
					$arr_raw_domains[$domain_id]=$obj_domain;
				}
				$arr_raw_domains[$domain_id]=$obj_domain;
			}
		}
		return $arr_raw_domains;
	}


	/**
	 * Validate a domain object
	 *
	 * @param stdClass Domain object
	 * @return stdClass Two objects - sanitized values, errors
	 * @author Eric Shelkie
	 * @since November 24, 2009
	 */
	static function validate_domain($obj_domain){
		global $allowedposttags;
		//print_r($obj_domain);
		$dc_instance=MiniMU::get_instance();
		$obj_errors=new stdClass();
		$obj_sanitized=new stdClass();
		$obj_return=new stdClass();
		$obj_return->sanitized=$obj_sanitized;
		$obj_return->errors=$obj_errors;

		
		// ID
		$obj_sanitized->id=intval($obj_domain->id);
		unset($obj_domain->id);
		
		// Is this the base domain?
		$obj_sanitized->is_base=(isset($obj_domain->is_base) && $obj_domain->is_base===true)?true:false;

		// Domain name
		if (trim($obj_domain->domain_name)!=''){
			// limit length
			$obj_domain->domain_name=substr($obj_domain->domain_name, 0, 100);
			
			preg_match("/^([a-z]([a-z0-9\-]*\.)+([a-z]{2}|aero|arpa|biz|com|coop|edu|gov|info|int|jobs|mil|museum|name|nato|net|org|pro|travel)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))(\/[a-z0-9_\-\.~]+)*(\/([a-z0-9_\-\.]*)(\?)?)?$/", $obj_domain->domain_name, $arr_matches);
			//echo "check: " . $obj_domain->domain_name . print_r($arr_matches, true);
			if (isset($arr_matches[1])){
				$obj_sanitized->domain_name=trim($arr_matches[1]);
			}else{
				$obj_sanitized->domain_name=clean_url($obj_domain->domain_name);
				$obj_errors->domain_name=array('message'=>'Please enter a valid domain name (eg. www.mydomain.com)');

			}
		}else{
			$obj_sanitized->domain_name='';
			//$obj_errors->domain_name=array('message'=>'Domain name is required (eg. www.mydomain.com)');
		}
		unset($obj_domain->domain_name);

		//print_r($dc_instance->arr_categories);
		// Category
		if (in_array($obj_domain->category_id, array_keys($dc_instance->arr_categories))){
			$obj_sanitized->category_id=$obj_domain->category_id;
		}else{
			$obj_sanitized->category_id=null;
		}
		unset($obj_domain->category_id);


		// Theme
		if (in_array($obj_domain->theme_name, array_keys($dc_instance->arr_themes))){
			$obj_sanitized->theme_name=$obj_domain->theme_name;
		}else{
			$obj_sanitized->theme_name='wordpress-default';
		}
		unset($obj_domain->theme_name);


		// Blog Title
		$obj_sanitized->blog_title=substr(wp_kses($obj_domain->blog_title, null), 0, 256);
		if ($obj_sanitized->blog_title=='') $obj_sanitized->blog_title=get_bloginfo( 'name' );
		unset($obj_domain->blog_title);


		// Tagline
		$obj_sanitized->tagline=substr(esc_html($obj_domain->tagline), 0, 512);
		unset($obj_domain->tagline);

		// Custom variables
		if (isset($obj_domain->custom_vars) && is_array($obj_domain->custom_vars) && count($obj_domain->custom_vars)>0){
			$obj_sanitized->custom_vars=array();
			foreach($obj_domain->custom_vars as $key=>$obj_value){
				// check custom variable name
				$key=trim(substr(MiniMU::sanitize_varname($key), 0, MiniMU::MAX_CUSTOM_VAR_NAME_LENGTH));
				if ($key){
					// sanitize custom variable value
					$obj_value_clean=new stdClass();
					$value=$obj_value->value;
					$value=wp_kses($value, $allowedposttags);
					if ($value){
						$obj_value_clean->value=$value;
						$obj_value_clean->type='textarea';
						$obj_sanitized->custom_vars[$key]=$obj_value_clean;
					}
				}	
			}
		}

		//print_r($obj_return);
		return $obj_return;
	}


	/**
	 * Add domain button handler
	 *
	 * @author Eric Shelkie
	 * @since November 24, 2009
	 */
	static function add_domain_click(){
		$dc_instance=MiniMU::get_instance();

		$new_domain_id=0;
		if (count($dc_instance->arr_domains)>0){
			$new_domain_id=max(array_keys($dc_instance->arr_domains))+1;
		}
		$new_domain=new stdClass();
		$new_domain->id=$new_domain_id;
		$new_domain->domain_name='';
		$new_domain->category_id=1;
		$new_domain->theme_name='wordpress-default';
		$new_domain->is_base=false;

		$dc_instance->arr_domains[$new_domain_id]=$new_domain;

		$obj_temp=new stdClass();
		$obj_temp->sanitized=MiniMU::get_empty_domain_object();
		$obj_temp->errors=new stdClass();
		$str_empty_domain=MiniMU::get_domain_row_html($obj_temp);
		echo json_encode(array("command"=>'minimu_add_empty_domain', 'args'=>$str_empty_domain));
		exit();
	}



	/**
	 * Get a default domain object;
	 *
	 * @return stdObject
	 * @author Eric Shelkie
	 * @since November 24, 2009
	 */
	static function	get_empty_domain_object(){
		$obj_empty_domain=new stdClass();
		$obj_empty_domain->is_base=false;
		$obj_empty_domain->id='new-' . time();
		$obj_empty_domain->domain_name='';
		$obj_empty_domain->category_id=1;
		$obj_empty_domain->theme_name='wordpress-default';
		$obj_empty_domain->blog_title=get_bloginfo( 'name' );
		$obj_empty_domain->tagline=get_bloginfo( 'description' );
		return $obj_empty_domain;
	}
	
	
	/**
	 * Validate variable name.
	 *
	 * @param string
	 * @return string
	 * @author Eric Shelkie
	 * @since December 1, 2009
	 */
	static function sanitize_varname($str_input){
		$str_output=preg_replace("/[^a-zA-Z0-9\-\_]/", '', $str_input);
		return $str_output;	
	}


	/**
	 * Generate the admin Menus
	 *
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	static function options_page(){
		$dc_instance=MiniMU::get_instance();

		?>
		<div class="wrap">
		<h2>MiniMU</h2>
		<?php if ($dc_instance->bln_settings_saved){ ?>
		<div class="updated fade" id="message" style="background-color: rgb(255, 251, 204);"><p><strong>Settings saved.</strong></p></div>
		<?php } ?>
		<form id="minimu-options-form" method="post" action="">


		<?php

		// Populate existing domain fields
		echo '<div id="sortcontainer">' . "\n";
		if (is_array($dc_instance->arr_domains)){
			foreach ($dc_instance->arr_domains as $obj_domain){
				//echo "check: " . print_R($obj_domain);
				$enabled=true;
				if ($obj_domain instanceof stdClass ){
					$obj_validated=MiniMU::validate_domain(clone $obj_domain);
					if (isset($obj_domain->is_base) && $obj_domain->is_base) $enabled=false;
					echo MiniMU::get_domain_row_html($obj_validated, array('disabled'=>!$enabled));
				}
			}
		}
		echo "</div>";




		$obj_temp=new stdClass();
		$obj_temp->sanitized=MiniMU::get_empty_domain_object();
		$obj_temp->errors=new stdClass();
		if (count($dc_instance->arr_domains)==0) echo MiniMU::get_domain_row_html($obj_temp);
		?>

		<input type="button" value="Add another domain" onclick="minimu_post_ajax({action:'minimu_add_domain_click'})"/>


		<?php /* <input type="hidden" name="action" value="update" />
		<input type="hidden" name="page_options" value="new_option_name,some_other_option,option_etc" />*/?>
		<input type="hidden" name="minimu-delete" id="minimu-delete" value="" />

		<p class="submit">
		<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
		</p>
		<?php //settings_fields( 'minimu-option-group' ); ?>
		<input type="hidden" id="minimu-action" name="minimu-action" value="update" />
		<input type="hidden" id="minimu-option-value" name="minimu-value" value="update" />
		</form>
		</div>
		<?
	}


	/**
	 * Get HTML for a single domain
	 *
	 * @param stdClass Domain data
	 * @param array $args
	 * @return string HTML
	 * @author Eric Shelkie
	 * @since November 23, 2009
	 */
	static function get_domain_row_html($obj_data, $args=array()){
		global $allowedposttags;
		$dc_instance=MiniMU::get_instance();
		$obj_domain=$obj_data->sanitized;
		$obj_error=$obj_data->errors;
		$disabled=false;
		$is_base=false;
		if (isset($args['disabled'])) $disabled=$args['disabled'];
		if (isset($obj_domain->is_base) ) $is_base=$obj_domain->is_base;


		
		ob_start();
		/*echo "<pre>";
		print_r($obj_data);
		echo "</pre>";*/
		?>
		<table id="minimu-tr-<?php echo $obj_domain->id;?>" class="minimu-domain-container sortable widget">
			<tr<?php if (isset($obj_error->domain_name)) echo ' class="error"' ?>>
				<td class="c0"><?php if (!$is_base){?><a class="remove" href="Javascript:{}" onclick="minimu_delete_domain('<?php echo $obj_domain->id;?>')">X</a><?php } ?></td>
				<td class="c1">Domain</td>
				<td class="c2">
					<?php if ($disabled){ ?>
						<input type="text" value="<?php echo $obj_domain->domain_name?>" disabled="disabled" />
						<input type="hidden" name="minimu-domain-name[<?php echo $obj_domain->id ?>]" value="<?php echo $obj_domain->domain_name?>" />
					<?php }else{ ?>
						<input type="text" name="minimu-domain-name[<?php echo $obj_domain->id ?>]" value="<?php echo $obj_domain->domain_name?>" />
					<?php } ?>
					<input type="hidden" name="minimu-domain-default[<?php echo $obj_domain->id ?>]" value="<?php echo ($is_base)?"true":"false" ?>" />
				</td>
				<td rowspan="50" class="sortable-handle c3">&nbsp;</td>
			</tr>
			<tr>
				<td class="c0"></td>
				<td class="c1">Category</td>
				<td><?php MiniMU::build_categories_select('minimu-catselect[' . $obj_domain->id . ']', $dc_instance->arr_categories, $obj_domain->category_id, $args);?></td>
			</tr>
			<tr>
				<td class="c0"></td>
				<td class="c1">Theme</td>
				<td><?php MiniMU::build_categories_select('minimu-themeselect[' . $obj_domain->id . ']', $dc_instance->arr_themes, $obj_domain->theme_name, $args);?></td>
			</tr>
			<tr>
				<td class="c0"></td>
				<td class="c1">Blog Title</td>
				<td><input type="text" name="minimu-blog-title[<?php echo $obj_domain->id ?>]" value="<?php echo $obj_domain->blog_title?>" <?php if ($disabled) echo 'disabled="disabled"' ?> /></td>
			</tr>
			<tr>
				<td class="c0"></td>
				<td class="c1">Tagline</td>
				<td><input type="text" name="minimu-tagline[<?php echo $obj_domain->id ?>]" value="<?php echo $obj_domain->tagline?>" <?php if ($disabled) echo 'disabled="disabled"' ?> /></td>
			</tr>
			<?php

			 if (isset($obj_domain->custom_vars) && is_array($obj_domain->custom_vars) && count($obj_domain->custom_vars)>0){
				 foreach ($obj_domain->custom_vars as $var_name=>$obj_var){
				 	$var_name=MiniMU::sanitize_varname($var_name);
				 	if ($var_name){
				 	?>
				<tr class="minimu-var-row">
					<td class="c0"><a class="remove" href="Javascript:{}">X</a></td>
					<td class="c1">
						<?php echo htmlspecialchars($var_name); ?>
					</td>
					<td>
						<?php
						switch($obj_var->type){
							/*default:
								echo '<input type="text" class="minimu-var-text" name="minimu-custom-vars[' . $obj_domain->id . '][' . $var_name . ']" value="' . wp_kses($obj_var->value, $allowedtags) . '" />';
								break;*/
								
							case 'textarea':
							default:
								echo '<textarea name="minimu-custom-vars[' . $obj_domain->id . '][' . $var_name . ']" class="minimu-var-textarea">' . $obj_var->value . '</textarea>';
								break;
								

						}
						?>
					</td>
				</tr>
			<?php
				 }
			  }
			}
			?>
			
			<tr class="minimu-add-var-row">
				<td colspan="2" style="padding: 3px 2px 10px 15px; text-align: right;">
					<a class="minimu-add-var-link" href="Javascript:{};" >More tokens</a>
					<select id="minimu-add-var[<?php echo $obj_domain->id ?>]" class="minimu-add-var-select">
						<option value="" >Select...</option>
						<?php
						foreach (MiniMU::get_custom_variable_names() as $var_name){
							$var_name=MiniMU::sanitize_varname($var_name);
							if (!isset($obj_domain->custom_vars) || ( isset($obj_domain->custom_vars) &&!array_key_exists($var_name, $obj_domain->custom_vars))){
								echo '<option value="' . $var_name . '">' . $var_name . "</option>\n";
							}
						}
						?>
						<option value="new" >Add new...</option>
					</select>
				</td>
				<td></td>
			</tr>
			
			<?php
			if (count((array)$obj_error)>0){?>
			<tr>
				<td colspan="3" class="error"><?php
				foreach ($obj_error as $arr_error){
					echo "- " . $arr_error['message'] . "<br />\n";
				}?>
			</tr>
				<?php
			}
			?>
		</table>
		<?php

		return ob_get_clean();
	}


	/**
	 * Get array of Wordpress categories
	 *
	 * @todo Check "child_of" - we should be able to point to sub-categories as well.
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	static function get_categories_array($parent=0){
		$dc_instance=MiniMU::get_instance();
		if(is_array($dc_instance->arr_categories)) return $dc_instance->arr_categories;
		$arr_temp = get_categories(array(
			'type'                     => 'post',
			'parent'         		   => $parent,
			'orderby'                  => 'name',
			'order'                    => 'ASC',
			'hide_empty'			   => false
			)
		);

		$dc_instance->arr_categories=array();
		
		// Let the user choose Blog Home as the category
		$obj_blank=new stdClass();
		$obj_blank->cat_ID=1;
		$obj_blank->name='Blog home';
		$dc_instance->arr_categories['1']=$obj_blank;
		
		// Put category objects into an associative array
		foreach ($arr_temp as $cat){
			if ($cat->cat_ID==1) continue;
			$dc_instance->arr_categories[$cat->cat_ID]=$cat;
		}


		return $dc_instance->arr_categories;
	}


	protected function get_themes_array(){
		if(is_array($this->arr_themes)) return $this->arr_themes;
		$arr_temp=get_themes();
		foreach ($arr_temp as $theme){
			if ($theme['Status']=='publish'){
				$obj_theme=(object)$theme;
				$this->arr_themes[sanitize_title_with_dashes($obj_theme->Name)]=$obj_theme;
			}
		}
	}
	
	
	/**
	 * Get an array of custom variable names in use
	 *
	 * @return array
	 * @author Eric Shelkie
	 * @since December 2, 2009
	 */
	static function get_custom_variable_names(){
		$dc_instance=MiniMU::get_instance();
		$arr_var_names=array();
		
		foreach ($dc_instance->arr_domains as $obj_domain){
			if (isset($obj_domain->custom_vars) && is_array($obj_domain->custom_vars)){
				foreach ($obj_domain->custom_vars as $name=>$value){
					$name=MiniMU::sanitize_varname($name);
					if (!in_array($name, $arr_var_names)) $arr_var_names[]=$name;
				}
			}
		}
		sort($arr_var_names);
		return $arr_var_names;
	}
	
	
	/**
	 * Get the slug for the current domain. Taken from linked category slug
	 *
	 * @return string
	 * @author Eric Shelkie
	 * @since December 1, 2009
	 */
	static function get_domain_slug(){
		$dc_instance=MiniMU::get_instance();
		$str_slug=null;
		
		$obj_options=$dc_instance->get_domain_options();
		if ($obj_options instanceof stdClass ){
			if ($obj_options->category_id){
				$obj_category=get_category($obj_options->category_id);
				if ($obj_category instanceof stdClass ){
					$str_slug=$obj_category->category_nicename;
				}
			}
		}
		return $str_slug;
	} 


	static function build_categories_select($control_name, $arr_options=array(), $selected_value=null, $args){
		$str_options='';
		//$pad=str_pad('',$depth*3,' ');
		$pad='';
		$disabled=false;
		if (isset($args['disabled'])) $disabled=$args['disabled'];
		
		foreach ($arr_options as $value=>$obj_category){
			if (isset($obj_category->cat_ID)) $value=$obj_category->cat_ID;
			$str_name=(isset($obj_category->name))?$obj_category->name:$obj_category->Name;
			$str_options.='<option value="' . htmlspecialchars($value) . '"';
			if ($value==$selected_value) $str_options.=' selected="selected"';
			$str_options.='>' . htmlspecialchars($pad . $str_name) . "</option>\n";
		}

		echo "<select name='$control_name' ";
		if ($disabled) echo 'disabled="disabled"';
		echo ">\n";
		echo $str_options;
		echo "</select>";

	}


	/**
	 * Get a list of domain/blogs configured
	 *
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	static function get_blog_list(){
		$dc_instance=MiniMU::get_instance();
		return $dc_instance->arr_domains;
	}


	/**
	 * Get a list of domain/blogs configured
	 *
	 * @param string Heading text
	 * @author Eric Shelkie
	 * @since November 22, 2009
	 */
	static function show_blog_list($args){
		$dc_instance=MiniMU::get_instance();
		$str_html='';
		$arr_domains=MiniMU::get_blog_list();
		if (is_array($arr_domains)){
			if (isset($args['heading'])) $str_html.= "<h2>" . htmlentities($args['heading']) . "</h2>\n<ul>";
			foreach ($arr_domains as $domain){
				if ($domain->is_base) $domain=$dc_instance->obj_base_domain;
				$str_html.= "<li";
				if ($domain->category_id==the_category_ID(false)) $str_html.= " class='active'";
				$str_html.= "><a href='http://" . MiniMU::get_token('domain_name', $domain->id) . "'>";
				
				// if set, use "link_text" for link text, otherwise, use blog title

				$link_text=$domain->blog_title;
				if (isset($args['link_text'])){
					if ($arg_text=MiniMU::get_token($args['link_text'], $domain->id)){
						$link_text=$arg_text;
					}
				}
				$str_html.=htmlspecialchars($link_text) . "</a></li>\n";
			}
			$str_html.= "</ul>\n";
		}
		echo $str_html;
	}

	
	/**
	 * Retrieve custom variable value
	 *
	 * @param string $var_name
	 * @param string $domain_id
	 * @return string value
	 * @author Eric Shelkie
	 * @since December 3, 2009
	 */
	static function get_token($var_name, $domain_id=null){
		$str_return='';
		$dc_instance=MiniMU::get_instance();
		
		$obj_options=MiniMU::get_domain_options($domain_id);
		
		if ($obj_options instanceof stdClass ){
			if ($var_name=='domain_name'){
				$domain_name=$obj_options->domain_name;
				$str_return=$domain_name;

				if (!$domain_name){
					$arr_categories=MiniMU::get_categories_array();
					$domain_name=$dc_instance->obj_base_domain->domain_name;
					
					// If a category has been selected, build a url for it, but only if blog uses structured permalinks
					if (isset($arr_categories[$obj_options->category_id]) && get_option('permalink_structure')){
						$obj_category=$arr_categories[$obj_options->category_id];

						if ($obj_category instanceof stdClass && isset($obj_category->slug) &&  $obj_category->slug!=""){
							$category_base=rtrim(get_option('category_base'),' /');
							if ($category_base=='') $category_base=MiniMU::DEFAULT_CATEGORY_BASE;
							if ($category_base) $domain_name=$domain_name . '/' . $category_base;
							$domain_name.= '/' . $obj_category->slug;
						}
					}
				}

				return $domain_name;
			}
			if (isset($obj_options->$var_name)) return $str_return=$obj_options->$var_name;
			if (isset($obj_options->custom_vars)){
				if (isset($obj_options->custom_vars[$var_name])){
					$obj_value=$obj_options->custom_vars[$var_name];
					$str_return=$obj_value->value;
				}
			}
		}
		//echo "<br /><br />";
		return $str_return;
	}
	
	
	/**
	 * Replace tokens with variable values
	 *
	 * @param string $strContent
	 * @return string
	 * @author Eric Shelkie
	 * @since March 9, 2010
	 */
	static function filter_replace_tokens($strContent){
		$arrCustomVars=MiniMU::get_custom_variable_names();
		foreach ($arrCustomVars as $varName){
			$strValue=MiniMU::get_token($varName);
			$strContent=str_replace("{*" . $varName . "*}", $strValue, $strContent);
		}
		return $strContent;
	}
	


}


// Get info for the base/default domain
// Need to do this before the plugin starts
$minimu_obj_base_domain=new stdClass();
$minimu_obj_base_domain->is_base=true;
$minimu_obj_base_domain->id=0;
$minimu_obj_base_domain->domain_name=preg_replace("/http[s]?\:\/\//i", '', get_option('siteurl'));
$minimu_obj_base_domain->category_id=1;
$minimu_obj_base_domain->theme_name=get_option('current_theme');
$minimu_obj_base_domain->blog_title=get_bloginfo('name');
$minimu_obj_base_domain->tagline=get_bloginfo('description');

// Initialize the plugin
$obj_MiniMU=new MiniMU();


?>